/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL04 for C using RDMSQL */
/* When you run this example, a new database is created with a single table
 * that has an index.  Ten rows are inserted in random order.
 * Finally all the data is read from the database using the index to order
 * the results.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown execptions
 * but good programming practices would dictate that this occur in
 * a full application.
 */

static void create_database(void)
{
    RSQL_ERRCODE rsqlCode;
    HCONN        hDbc;
    HSTMT        hStmt;
    int16_t      ii;

    const char *cmds[] = {
        "CREATE DATABASE SQL04_db",
        "CREATE TABLE IndTab (id INTEGER PRIMARY KEY)",
        "COMMIT"
    };

    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    (void) rsqlExecDirect(hStmt, "DROP DATABASE SQL04_db");
    (void) rsqlExecDirect(hStmt, "COMMIT");

    for (ii = 0; ii < sizeof(cmds) / sizeof(cmds[0]); ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the SQL04 database\n");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, "SQL04_db");

    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    int16_t           ii;

    create_database();

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Open the database in exclusive mode */
    (void) rsqlOpenDB(hDbc, "SQL04_db", "x");

    srand((unsigned) time(NULL));

    /* Insert ten rows into the table with random data */
    for (ii = 0; ii < 10; ii++)
    {
        char commandText[80];

        sprintf(commandText, "INSERT INTO IndTab VALUES %d", rand() % 100);
        rsqlCode = rsqlExecDirect(hStmt, commandText);
    }

    /* Scan through all the existing rows using the index for order */
    (void) rsqlExecDirect(hStmt, "SELECT id FROM IndTab order by id");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%3d\n", result[0].vt.lv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc, "SQL04_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

